@@ -106,7 +106,7 @@ enum_string_to_value (GType type,
 		}
 		g_warning ("Unhandled value '%s' for enum '%s'",
 			   string, G_FLAGS_CLASS_TYPE_NAME (eclass));
-		g_assert_not_reached ();
+		return 0;
 	}
 
 	return value->value;
@@ -485,6 +485,9 @@ cc_background_xml_load_xml (CcBackgroundXml *xml,
 {
 	g_return_val_if_fail (CC_IS_BACKGROUND_XML (xml), FALSE);
 
+	if (g_file_test (filename, G_FILE_TEST_IS_REGULAR) == FALSE)
+		return NULL;
+
 	return cc_background_xml_load_xml_internal (xml, filename, FALSE);
 }
 
