@@ -73,6 +73,8 @@ struct _CcPrintersPanelPrivate
 
   GPermission *permission;
 
+  GSettings *lockdown_settings;
+
   PpNewPrinterDialog *pp_new_printer_dialog;
 
   gpointer dummy;
@@ -81,6 +83,7 @@ struct _CcPrintersPanelPrivate
 static void actualize_jobs_list (CcPrintersPanel *self);
 static void actualize_printers_list (CcPrintersPanel *self);
 static void actualize_allowed_users_list (CcPrintersPanel *self);
+static void actualize_sensitivity (gpointer user_data);
 static void printer_disable_cb (GObject *gobject, GParamSpec *pspec, gpointer user_data);
 static void printer_set_default_cb (GtkToggleButton *button, gpointer user_data);
 
@@ -149,6 +152,12 @@ cc_printers_panel_dispose (GObject *object)
       priv->builder = NULL;
     }
 
+  if (priv->lockdown_settings)
+    {
+      g_object_unref (priv->lockdown_settings);
+      priv->lockdown_settings = NULL;
+    }
+
   G_OBJECT_CLASS (cc_printers_panel_parent_class)->dispose (object);
 }
 
@@ -203,16 +212,14 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
   CcPrintersPanelPrivate *priv;
   CcPrintersPanel        *self = (CcPrintersPanel*) user_data;
   GtkTreeModel           *model;
+  cups_ptype_t            type = 0;
   GtkTreeIter             iter;
   GtkWidget              *widget;
-  gboolean                test_page_command_available = FALSE;
   gboolean                sensitive;
   gboolean                is_local = TRUE;
   gchar                  *printer_make_and_model = NULL;
   gchar                  *printer_model = NULL;
   gchar                  *reason = NULL;
-  gchar                 **available_commands = NULL;
-  gchar                  *printer_commands = NULL;
   gchar                 **printer_reasons = NULL;
   gchar                  *marker_types = NULL;
   gchar                  *printer_name = NULL;
@@ -319,8 +326,6 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
             reason = priv->dests[id].options[i].value;
           else if (g_strcmp0 (priv->dests[priv->current_dest].options[i].name, "marker-types") == 0)
             marker_types = priv->dests[priv->current_dest].options[i].value;
-          else if (g_strcmp0 (priv->dests[priv->current_dest].options[i].name, "printer-commands") == 0)
-            printer_commands = priv->dests[priv->current_dest].options[i].value;
           else if (g_strcmp0 (priv->dests[priv->current_dest].options[i].name, "printer-make-and-model") == 0)
             printer_make_and_model = priv->dests[priv->current_dest].options[i].value;
           else if (g_strcmp0 (priv->dests[priv->current_dest].options[i].name, "printer-uri-supported") == 0)
@@ -492,7 +497,6 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
 
       if (printer_type)
         {
-          cups_ptype_t type;
           type = atoi (printer_type);
           is_local = !(type & (CUPS_PRINTER_REMOTE | CUPS_PRINTER_IMPLICIT));
         }
@@ -543,22 +547,6 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
       gtk_widget_set_sensitive (widget, sensitive);
 
 
-      if (printer_commands)
-        {
-          available_commands = g_strsplit (printer_commands, ",", -1);
-          for (i = 0; i < g_strv_length (available_commands); i++)
-            {
-              if (g_strcmp0 (available_commands[i], "PrintSelfTestPage") == 0)
-                test_page_command_available = TRUE;
-            }
-          g_strfreev (available_commands);
-        }
-
-      widget = (GtkWidget*)
-        gtk_builder_get_object (priv->builder, "print-test-page-button");
-      gtk_widget_set_sensitive (widget, test_page_command_available);
-
-
       widget = (GtkWidget*)
         gtk_builder_get_object (priv->builder, "supply-drawing-area");
       gtk_widget_set_size_request (widget, -1, SUPPLY_BAR_HEIGHT);
@@ -600,19 +588,6 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
         }
       else
         gtk_label_set_text (GTK_LABEL (widget), EMPTY_TEXT);
-
-
-      widget = (GtkWidget*)
-        gtk_builder_get_object (priv->builder, "printer-options-button");
-      gtk_widget_set_sensitive (widget, TRUE);
-
-      widget = (GtkWidget*)
-        gtk_builder_get_object (priv->builder, "printer-jobs-button");
-      gtk_widget_set_sensitive (widget, TRUE);
-
-      widget = (GtkWidget*)
-        gtk_builder_get_object (priv->builder, "printer-icon");
-      gtk_widget_set_sensitive (widget, TRUE);
     }
   else
     {
@@ -653,24 +628,10 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
       widget = (GtkWidget*)
         gtk_builder_get_object (priv->builder, "printer-jobs-label");
       gtk_label_set_text (GTK_LABEL (widget), "");
-
-      widget = (GtkWidget*)
-        gtk_builder_get_object (priv->builder, "print-test-page-button");
-      gtk_widget_set_sensitive (widget, FALSE);
-
-      widget = (GtkWidget*)
-        gtk_builder_get_object (priv->builder, "printer-options-button");
-      gtk_widget_set_sensitive (widget, FALSE);
-
-      widget = (GtkWidget*)
-        gtk_builder_get_object (priv->builder, "printer-jobs-button");
-      gtk_widget_set_sensitive (widget, FALSE);
-
-      widget = (GtkWidget*)
-        gtk_builder_get_object (priv->builder, "printer-icon");
-      gtk_widget_set_sensitive (widget, FALSE);
     }
 
+  actualize_sensitivity (self);
+
   widget = (GtkWidget*)
     gtk_builder_get_object (priv->builder, "job-release-button");
   gtk_widget_set_sensitive (widget, FALSE);
@@ -1954,62 +1915,159 @@ printer_remove_cb (GtkToolButton *toolbutton,
 }
 
 static void
-printer_maintenance_cb (GtkButton *button,
-                        gpointer   user_data)
+test_page_cb (GtkButton *button,
+              gpointer   user_data)
 {
   CcPrintersPanelPrivate  *priv;
   CcPrintersPanel         *self = (CcPrintersPanel*) user_data;
-  ipp_t                   *response = NULL;
+  cups_ptype_t             type = 0;
+  const gchar             *printer_type = NULL;
   gchar                   *printer_name = NULL;
+  gint                     i;
 
   priv = PRINTERS_PANEL_PRIVATE (self);
 
   if (priv->current_dest >= 0 &&
       priv->current_dest < priv->num_dests &&
       priv->dests != NULL)
-    printer_name = priv->dests[priv->current_dest].name;
+    {
+      printer_name = priv->dests[priv->current_dest].name;
+      printer_type = cupsGetOption ("printer-type",
+                                    priv->dests[priv->current_dest].num_options,
+                                    priv->dests[priv->current_dest].options);
+      if (printer_type)
+        type = atoi (printer_type);
+    }
 
   if (printer_name)
     {
-      if ((GtkButton*) gtk_builder_get_object (priv->builder,
-                                               "print-test-page-button")
-                       == button)
+      const gchar  *const dirs[] = { "/usr/share/cups",
+                                     "/usr/local/share/cups",
+                                     NULL };
+      const gchar  *testprint[] = { "%s/data/testprint",
+                                    "%s/data/testprint.ps",
+                                    NULL };
+      const gchar **pattern;
+      const gchar  *datadir = NULL;
+      http_t       *http = NULL;
+      gchar        *printer_uri = NULL;
+      gchar        *filename = NULL;
+      gchar        *resource = NULL;
+      ipp_t        *response;
+      ipp_t        *request;
+
+      if ((datadir = getenv ("CUPS_DATADIR")) != NULL)
+        {
+          for (pattern = testprint; *pattern != NULL; pattern++)
+            {
+              filename = g_strdup_printf (*pattern, datadir);
+              if (g_access (filename, R_OK) == 0)
+                break;
+              else
+                {
+                  g_free (filename);
+                  filename = NULL;
+                }
+            }
+        }
+      else
+        {
+          for (i = 0; (datadir = dirs[i]) != NULL && filename == NULL; i++)
+            {
+              for (pattern = testprint; *pattern != NULL; pattern++)
+                {
+                  filename = g_strdup_printf (*pattern, datadir);
+                  if (g_access (filename, R_OK) == 0)
+                    break;
+                  else
+                    {
+                      g_free (filename);
+                      filename = NULL;
+                    }
+                }
+            }
+        }
+
+      if (filename)
+        {
+          if (type & CUPS_PRINTER_CLASS)
+            {
+              printer_uri = g_strdup_printf ("ipp://localhost/classes/%s", printer_name);
+              resource = g_strdup_printf ("/classes/%s", printer_name);
+            }
+          else
+            {
+              printer_uri = g_strdup_printf ("ipp://localhost/printers/%s", printer_name);
+              resource = g_strdup_printf ("/printers/%s", printer_name);
+            }
+
+          http = httpConnectEncrypt (cupsServer (), ippPort (), cupsEncryption ());
+          if (http)
+            {
+              request = ippNewRequest (IPP_PRINT_JOB);
+              ippAddString (request, IPP_TAG_OPERATION, IPP_TAG_URI,
+                            "printer-uri", NULL, printer_uri);
+              ippAddString (request, IPP_TAG_OPERATION, IPP_TAG_NAME,
+                            "requesting-user-name", NULL, cupsUser ());
+              ippAddString (request, IPP_TAG_OPERATION, IPP_TAG_NAME,
+              /* Translators: Name of job which makes printer to print test page */
+                            "job-name", NULL, _("Test page"));
+              response = cupsDoFileRequest (http, request, resource, filename);
+              httpClose (http);
+            }
+
+          g_free (filename);
+          g_free (printer_uri);
+          g_free (resource);
+        }
+      else
         {
           response = execute_maintenance_command (printer_name,
                                                   "PrintSelfTestPage",
           /* Translators: Name of job which makes printer to print test page */
                                                   _("Test page"));
         }
-      else if ((GtkButton*) gtk_builder_get_object (priv->builder,
-                                                    "clean-print-heads-button")
-                            == button)
-        response = execute_maintenance_command (printer_name,
-                                                "Clean all",
-          /* Translators: Name of job which makes printer to clean its heads */
-                                                _("Clean print heads"));
+
       if (response)
         {
           if (response->state == IPP_ERROR)
-            /* Translators: An error has occured during execution of a CUPS maintenance command */
-            g_warning (_("An error has occured during a maintenance command."));
-          ippDelete(response);
+            g_warning ("An error has occured during printing of test page.");
+          ippDelete (response);
         }
     }
 }
 
 static void
-on_permission_changed (GPermission *permission,
-                       GParamSpec  *pspec,
-                       gpointer     data)
+actualize_sensitivity (gpointer user_data)
 {
   CcPrintersPanelPrivate  *priv;
-  CcPrintersPanel         *self = (CcPrintersPanel*) data;
+  CcPrintersPanel         *self = (CcPrintersPanel*) user_data;
+  cups_ptype_t             type = 0;
   GtkWidget               *widget;
   gboolean                 is_authorized;
+  gboolean                 is_discovered = FALSE;
+  gboolean                 printer_selected;
+  gint                     i;
 
   priv = PRINTERS_PANEL_PRIVATE (self);
 
-  is_authorized = g_permission_get_allowed (G_PERMISSION (priv->permission));
+  is_authorized = g_permission_get_allowed (G_PERMISSION (priv->permission)) &&
+    !g_settings_get_boolean (priv->lockdown_settings, "disable-print-setup");
+
+  printer_selected = priv->current_dest >= 0 &&
+                     priv->current_dest < priv->num_dests &&
+                     priv->dests != NULL;
+
+  if (printer_selected)
+    for (i = 0; i < priv->dests[priv->current_dest].num_options; i++)
+      {
+        if (g_strcmp0 (priv->dests[priv->current_dest].options[i].name, "printer-type") == 0)
+          {
+            type = atoi (priv->dests[priv->current_dest].options[i].value);
+            is_discovered = type & CUPS_PRINTER_DISCOVERED;
+            break;
+          }
+      }
 
   widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "printer-add-button");
   gtk_widget_set_sensitive (widget, is_authorized);
@@ -2018,7 +2076,7 @@ on_permission_changed (GPermission *permission,
   gtk_widget_set_sensitive (widget, is_authorized);
 
   widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "printer-remove-button");
-  gtk_widget_set_sensitive (widget, is_authorized);
+  gtk_widget_set_sensitive (widget, !is_discovered && is_authorized);
 
   widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "printer-disable-switch");
   gtk_widget_set_sensitive (widget, is_authorized);
@@ -2031,6 +2089,45 @@ on_permission_changed (GPermission *permission,
 
   widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "allowed-user-remove-button");
   gtk_widget_set_sensitive (widget, is_authorized);
+
+  widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "print-test-page-button");
+  gtk_widget_set_sensitive (widget, printer_selected);
+
+  widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "printer-options-button");
+  gtk_widget_set_sensitive (widget, printer_selected);
+
+  widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "printer-jobs-button");
+  gtk_widget_set_sensitive (widget, printer_selected);
+
+  widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "printer-icon");
+  gtk_widget_set_sensitive (widget, printer_selected);
+}
+
+static void
+on_permission_changed (GPermission *permission,
+                       GParamSpec  *pspec,
+                       gpointer     data)
+{
+  actualize_sensitivity (data);
+}
+
+static void
+on_lockdown_settings_changed (GSettings  *settings,
+                              const char *key,
+                              gpointer    user_data)
+{
+  CcPrintersPanelPrivate  *priv;
+  CcPrintersPanel         *self = (CcPrintersPanel*) user_data;
+
+  if (g_str_equal (key, "disable-print-setup") == FALSE)
+    return;
+
+  priv = PRINTERS_PANEL_PRIVATE (self);
+
+  gtk_widget_set_sensitive (priv->lock_button,
+    !g_settings_get_boolean (priv->lockdown_settings, "disable-print-setup"));
+
+  on_permission_changed (priv->permission, NULL, user_data);
 }
 
 static void
@@ -2171,7 +2268,7 @@ cc_printers_panel_init (CcPrintersPanel *self)
 
   widget = (GtkWidget*)
     gtk_builder_get_object (priv->builder, "print-test-page-button");
-  g_signal_connect (widget, "clicked", G_CALLBACK (printer_maintenance_cb), self);
+  g_signal_connect (widget, "clicked", G_CALLBACK (test_page_cb), self);
 
   widget = (GtkWidget*)
     gtk_builder_get_object (priv->builder, "back-button-1");
@@ -2189,6 +2286,12 @@ cc_printers_panel_init (CcPrintersPanel *self)
     gtk_builder_get_object (priv->builder, "printer-options-button");
   g_signal_connect (widget, "clicked", G_CALLBACK (switch_to_options_cb), self);
 
+  priv->lockdown_settings = g_settings_new ("org.gnome.desktop.lockdown");
+  g_signal_connect (priv->lockdown_settings,
+                    "changed",
+                    G_CALLBACK (on_lockdown_settings_changed),
+                    self);
+
 
   /* Set junctions */
   widget = (GtkWidget*)
@@ -2224,7 +2327,7 @@ cc_printers_panel_init (CcPrintersPanel *self)
 
   /* Add unlock button */
   priv->permission = (GPermission *)polkit_permission_new_sync (
-    "org.opensuse.cupspkhelper.mechanism.printeraddremove", NULL, NULL, NULL);
+    "org.opensuse.cupspkhelper.mechanism.all-edit", NULL, NULL, NULL);
   if (priv->permission != NULL)
     {
       widget = cc_lock_button_new (priv->permission);
